﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Persistence.Data;

namespace HeuristicLab.Persistence {
  [Transformer("97B5CFC8-CDFA-4EB5-B4CD-5B3CFA5CD844", 219)]
  internal sealed class KeyValuePairTransformer : TransformerBase {
    public override bool CanTransformType(Type type) {
      return type.IsGenericType && (type.GetGenericTypeDefinition() == typeof(KeyValuePair<,>));
    }
    public override PersistenceData ToData(object o, PersistenceMapper mapper) {
      var data = new ArrayData<uint>(Id);
      var type = o.GetType();
      data.TypeId = mapper.GetTypeId(type);
      data.Values = new uint[2];
      data.Values[0] = mapper.GetDataId(type.GetProperty("Key").GetValue(o, null));
      data.Values[1] = mapper.GetDataId(type.GetProperty("Value").GetValue(o, null));
      return data;
    }

    public override object ToObject(PersistenceData data, PersistenceMapper mapper) {
      var composite = (ArrayData<uint>)data;
      var key = mapper.GetObject(composite.Values[0]);
      var value = mapper.GetObject(composite.Values[1]);
      return Activator.CreateInstance(mapper.GetType(composite.TypeId), key, value);
    }
  }
}
